<?php
namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities\Util;
use Teknisa\MAN\Utilities\Resposta; 

class TransmissaoNfceService {
    
    public $entityManager;
    
   	public function __construct(\Doctrine\ORM\EntityManager $entityManager, Util $util)  {
		$this->entityManager = $entityManager;
		$this->util = $util;
	}
	
	public function executarQuery($query, $params) {
	    
	    $retorno = new Resposta();
	    $retorno->setParameters($params);
	    try {
	        $resultado = $this->entityManager->getConnection()->fetchAll($query, $params);
	        $retorno->setResult($resultado);
	        $retorno->setMessage("Query executada com sucesso. Linhas retornadas: ".$retorno->getLinhasResult());
	        $retorno->setStatus(Resposta::SUCESSO);
	    } catch (\Exception $e){
            $retorno->setMessage($e->getMessage());
            $retorno->setError(true);
            $retorno->setStatus(Resposta::ERRO);
        }
        return $retorno;
	}
	
	public function runQuery($query, $params) {
	    return $this->entityManager->getConnection()->executeQuery($query, $params);
	}
	
	public function getDuplicidades($row) {

        $parametros = $this->gerarParametros(['CDFILIAL', 'DATAINI', 'DATAFIN'], $row);
        $resposta = $this->executarQuery(self::ADQGETDUPLICIDADES, $parametros);
        if ($resposta->error()) {
            throw new \Exception("Erro ao buscar duplicidades: ".$resposta->getMessage());
        } 
        
        return $resposta->getResult();
        
    }
    
    public function iniciarTentativas($cdfilial, $cdcaixa, $nrseqvenda, $nrorg) {
        $this->entityManager->beginTransaction();
        try{
            $parametros = [
                ":CDFILIAL" => $cdfilial,
                ":CDCAIXA" => $cdcaixa,
                ":NRSEQVENDA" => $nrseqvenda,
                ":NRORG" => $nrorg
            ];
            $this->entityManager->getConnection()->executeQuery(self::UPDATENRENVIOSEFAZ, $parametros);
            
            $this->entityManager->commit();
            return true;
        }catch (\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function getNotasInvalidacao($row) {

        $parametros = $this->gerarParametros(['CDFILIAL', 'DATAINI', 'DATAFIN', 'CDCAIXA'], $row);
        $resposta = $this->executarQuery(self::ADQNUMERACAO_FULL, $parametros);
        
        if ($resposta->error()) {
            throw new \Exception("Erro ao buscar notas: ".$resposta->getMessage());
        } 
        
        return $resposta->getResult();
        
    }
    
    public function updateVendaNFCE($parametros) {
        // $types['DSARQXMLNFCE'] = \Doctrine\DBAL\Types\Type::BINARY;
        $this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_NFCE, $parametros);
    }
    
    private function gerarParametros(array $parametros, $objeto) {
        
        $parametrosQuery = array();
        
        foreach ($parametros as $key) {
            if (empty($objeto[$key])) {
                if ($key == 'CDFILIAL' || $key == 'CDCAIXA')
                    $parametrosQuery[':'.$key] = 'T';
                else
                    $parametrosQuery[':'.$key] = null;
            } else {
                if (is_array($objeto[$key]))
                    $parametrosQuery[':'.$key] = implode("_", $objeto[$key]);
                else 
                    $parametrosQuery[':'.$key] = $objeto[$key];
            }
        }
        
        return $parametrosQuery;
    }
	
	const BUSCAR_NOTAS = "
    WITH FC AS
  (SELECT DISTINCT F.*, FLC.CDCAIXA, FLC.NMCAIXA, FLC.CDLOJA
      FROM V_FILIALLOJACAIXA FLC, (SELECT CDFILIAL, NMFILIAL, CDINSCMUNI, CDINSCESTA, NMRAZSOCFILI, CDINSCESTA AS CDINSCESTA2, NRINSJURFILI
                                     FROM FILIAL
                                    WHERE ((INSTR(:CDFILIAL, CDFILIAL) <> 0) OR (:CDFILIAL        = 'T'))) F
     WHERE FLC.CDFILIAL = F.CDFILIAL
      AND ((INSTR(:CDCAIXA, FLC.CDFILIAL||'_'||FLC.CDCAIXA    ) <> 0) OR (:CDCAIXA         = 'T')))

SELECT *  FROM (SELECT
      SUBSTR('Nota Fiscal: ' || V.NRNOTAFISCALCE || ' Caixa: ' || V.CDCAIXA, 1, 33) AS NOTA,
      '      Abrir .txt' AS XMLNOTA, V.NRENVIOSEFAZ, V.DTABERTUR, V.CDOPERADOR, V.IDTPEMISVEND, V.NRORG,
      V.CDFILIAL, V.CDCAIXA, V.CDLOJA, V.NRSEQVENDA, V.DTENTRVENDA, V.NRNOTAFISCALCE, V.DTEMISSAONFCE, V.NRRECIBONFCE, V.NRPROTOCOLONFCE,
      FC.NMFILIAL, V.CDSERIENFCE, V.IDSITUVENDA, V.IDTPAMBNFCE, 'U' AS IDCHECK, V.IDSTATUSNFCE, V.CDEMPRESA, V.IDMODOPERACNFCE,
      FC.NMCAIXA, V.DSOBSSTATUSNFCE, V.NRACESSONFCE, FC.CDINSCMUNI, FC.CDINSCESTA, V.NRLANCTONFCE, P.NRCERTDIGNFCE, V.DTHRPROTOCONFCE,
      V.IDXMLVALIDNFCE, V.NRINSCRCONS, V.DSARQXMLNFCE, A.VRTOTALVENDA, S.SGESTADO, S.CDESTADOIBGE , V.DSQRCODENFCE, P.CDVERSAOXMLNFCE, V.DTVENDA,
      CASE WHEN V.IDSITUVENDA = 'C' AND V.IDSTATUSNFCE <> 'C' THEN 'S' ELSE 'N' END AS RED,
      CASE WHEN V.IDSITUVENDA <> 'C' THEN 'S' ELSE 'N' END AS BLACK,
      P.CDCODSCONSPROD, P.CDIDTOKENPROD, P.CDFILIAL AS CDFILIAL2, P.CDCODSCONSHOMO, P.CDIDTOKENHOMO, FC.NMRAZSOCFILI, FC.NRINSJURFILI, FC.CDINSCESTA2,
      P.CDURLWSNFC
    FROM
      VENDA V, PARAVEND P, ENDEFILI E, ESTADO S, FC,
      (SELECT
        V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND ,0) + NVL(I.VRACRITVEND,0)) AS VRTOTALVENDA
      FROM
        VENDA V, ITEMVENDA I, FC
      WHERE
        (V.CDFILIAL     = I.CDFILIAL    ) AND
        (V.CDCAIXA      = I.CDCAIXA     ) AND
        (V.NRSEQVENDA   = I.NRSEQVENDA  ) AND
        (V.CDFILIAL = FC.CDFILIAL) AND
        (V.CDCAIXA = FC.CDCAIXA) AND
        (V.CDLOJA = FC.CDLOJA) AND
        (V.IDSITUVENDA  <> 'D') AND
        (V.IDTPAMBNFCE  = :IDTPAMBNFCE) AND
        (V.IDTPEMISVEND = 'N'           ) AND
        (INSTR(:SITUACAO, V.IDSTATUSNFCE) <> 0) AND
        ((INSTR(:IDMODOPERACNFCE, SUBSTR(V.NRACESSONFCE,35,1)) <> 0) OR (:IDMODOPERACNFCE = 'T')) AND
        ((INSTR(:NRNOTAFISCALCE, V.NRNOTAFISCALCE              ) <> 0) OR (:NRNOTAFISCALCE        = 'T')) AND
        ((INSTR(:IDSITUVENDA, V.IDSITUVENDA        ) <> 0) OR (:IDSITUVENDA     = 'T')) AND
        ((V.NRINSCRCONS IS NOT NULL AND :IDENVIACPF = 'S') OR (V.NRINSCRCONS IS NULL AND :IDENVIACPF = 'N') OR (:IDENVIACPF = 'T')) AND
        (V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN) AND
                      (V.NRACESSONFCE IS NOT NULL)
        GROUP BY V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA) A
    WHERE
      (V.CDFILIAL     = A.CDFILIAL    ) AND
      (V.CDCAIXA      = A.CDCAIXA     ) AND
      (V.NRSEQVENDA   = A.NRSEQVENDA  ) AND
      (V.CDFILIAL     = P.CDFILIAL    ) AND
      (E.CDFILIAL     = FC.CDFILIAL    ) AND
      (E.IDTPENDEFILI = 'P'           ) AND
      (S.CDPAIS       = E.CDPAIS      ) AND
      (S.SGESTADO     = E.SGESTADO    ) AND
      (V.CDFILIAL = FC.CDFILIAL) AND
      (V.CDCAIXA = FC.CDCAIXA) AND
      (V.CDLOJA = FC.CDLOJA)

    ORDER BY
      V.CDFILIAL, V.CDCAIXA, V.NRNOTAFISCALCE)
      WHERE (CDFILIAL||CDCAIXA||DTENTRVENDA||NRNOTAFISCALCE||CDSERIENFCE||VRTOTALVENDA||IDSITUVENDA||NRINSCRCONS||NRACESSONFCE like :ALLFIEDLS  OR :ALLFIEDLS = 'T' )";
	
	const BUSCAR_NOTAS_ERR = "
	    SELECT
          SUBSTR('Nota Fiscal: ' || V.NRNOTAFISCALCE || ' Caixa: ' || V.CDCAIXA, 1, 33) AS NOTA,
          '      Abrir .txt' AS XMLNOTA, V.NRORG, V.DTABERTUR, V.CDOPERADOR, V.IDTPEMISVEND,
          V.CDFILIAL, V.CDCAIXA, V.CDLOJA, V.NRSEQVENDA, V.DTENTRVENDA, V.NRNOTAFISCALCE, V.DTEMISSAONFCE, V.NRRECIBONFCE,
          F.NMFILIAL, V.CDSERIENFCE, V.IDSITUVENDA, V.IDTPAMBNFCE, 'U' AS IDCHECK, V.IDSTATUSNFCE, V.CDEMPRESA, V.IDMODOPERACNFCE,
          C.NMCAIXA, V.DSOBSSTATUSNFCE, V.NRACESSONFCE, F.CDINSCMUNI, F.CDINSCESTA, V.NRLANCTONFCE, P.NRCERTDIGNFCE, V.DTHRPROTOCONFCE, IP.VRTOTTRIBIBPT,
          V.IDXMLVALIDNFCE, V.NRINSCRCONS, V.DSARQXMLNFCE, A.VRTOTALVENDA, S.SGESTADO, S.CDESTADOIBGE , V.DSQRCODENFCE, P.CDVERSAOXMLNFCE, V.NRENVIOSEFAZ
        FROM
          VENDA V, FILIAL F, PARAVEND P, CAIXA C, ENDEFILI E, ESTADO S, ITVENDAIMPOS IP,
        														(SELECT
        															 V.CDFILIAL, I.NRSEQUITVEND, V. CDCAIXA, V.NRSEQVENDA, SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND ,0) + NVL(I.VRACRITVEND,0)) AS VRTOTALVENDA
        														 FROM
        														 	 VENDA V, ITEMVENDA I
        														 WHERE
        														 	 (V.CDFILIAL     = I.CDFILIAL    ) AND
        														 	 (V.CDCAIXA      = I.CDCAIXA     ) AND
        															 (V.NRSEQVENDA   = I.NRSEQVENDA  ) AND
        															 (V.IDSITUVENDA  <> 'D'          ) AND
        															 (V.IDTPAMBNFCE  = :IDTPAMBNFCE) AND
                                                                     (V.IDTPEMISVEND = 'N'           ) AND
        															 (INSTR(:SITUACAO, V.IDSTATUSNFCE) <> 0) AND
        															 ((INSTR(:CDFILIAL, V.CDFILIAL              ) <> 0) OR (:CDFILIAL        = 'T')) AND
        															 ((INSTR(:CDCAIXA, V.CDFILIAL||V.CDCAIXA    ) <> 0) OR (:CDCAIXA         = 'T')) AND
        															 ((INSTR(:IDMODOPERACNFCE, SUBSTR(V.NRACESSONFCE,35,1)) <> 0) OR (:IDMODOPERACNFCE = 'T')) AND
        															 ((INSTR(:IDSITUVENDA, V.IDSITUVENDA        ) <> 0) OR (:IDSITUVENDA     = 'T')) AND
        														 	 ((INSTR(:NRINSCRCONS, V.NRINSCRCONS        ) <> 0) OR (:NRINSCRCONS     = 'T')) AND
        															 (V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN) AND
                                                                     (V.NRACESSONFCE IS NOT NULL)
        														  GROUP BY V.CDFILIAL, I.NRSEQUITVEND, V.CDCAIXA, V.NRSEQVENDA) A
        WHERE
          (V.CDFILIAL     = F.CDFILIAL    ) AND
          (V.CDFILIAL     = A.CDFILIAL    ) AND
          (V.CDCAIXA      = A.CDCAIXA     ) AND
          (V.NRSEQVENDA   = A.NRSEQVENDA  ) AND
          (V.CDFILIAL     = P.CDFILIAL    ) AND
          (V.CDFILIAL     = C.CDFILIAL    ) AND
          (V.CDCAIXA      = C.CDCAIXA     ) AND
          (V.IDSITUVENDA  <> 'D'          ) AND
          (V.IDTPAMBNFCE  = :IDTPAMBNFCE) AND
          (V.IDTPEMISVEND = 'N'           ) AND
          (E.CDFILIAL     = F.CDFILIAL    ) AND
          (E.IDTPENDEFILI = 'P'           ) AND
          (S.CDPAIS       = E.CDPAIS      ) AND
          (S.SGESTADO     = E.SGESTADO    ) AND
          (IP.CDFILIAL      = V.CDFILIAL    ) AND
          (IP.CDCAIXA       = V.CDCAIXA     ) AND
          (IP.NRSEQUITVEND  = A.NRSEQUITVEND) AND
          (IP.NRSEQVENDA    = V.NRSEQVENDA  ) AND
          (INSTR(:SITUACAO, V.IDSTATUSNFCE) <> 0) AND
          ((INSTR(:CDFILIAL, V.CDFILIAL              ) <> 0) OR (:CDFILIAL        = 'T')) AND   
          ((INSTR(:CDCAIXA, V.CDFILIAL||V.CDCAIXA    ) <> 0) OR (:CDCAIXA         = 'T')) AND
          ((INSTR(:IDMODOPERACNFCE, SUBSTR(V.NRACESSONFCE,35,1)) <> 0) OR (:IDMODOPERACNFCE = 'T')) AND
          ((INSTR(:IDSITUVENDA, V.IDSITUVENDA        ) <> 0) OR (:IDSITUVENDA     = 'T')) AND
          ((INSTR(:NRINSCRCONS, V.NRINSCRCONS        ) <> 0) OR (:NRINSCRCONS     = 'T')) AND
          (V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN) AND
          (V.NRACESSONFCE IS NOT NULL)
        ORDER BY
          V.CDFILIAL, V.CDCAIXA, V.NRNOTAFISCALCE
	";
	
	const ADQNUMERACAO = "
        SELECT I.CDFILIAL, I.CDCAIXA, I.NRNOTAFISCALCE, I.CDSERIENFCE, I.IDTPAMBNFCE,
          F.CDEMPRESA, F.CDINSCESTA, F.CDINSCMUNI, P.NRCERTDIGNFCE, I.IDSTATUSNFCE,
          I.DSOBSINUTNFCE, P.CDVERSAOXMLNFCE
        FROM
          INUTILIZANFCE I, FILIAL F, PARAVEND P
        WHERE
          (I.CDFILIAL    = :CDFILIAL   ) AND
          ((INSTR(:CDCAIXA, I.CDFILIAL||I.CDCAIXA) <> 0) OR (:CDCAIXA = 'T')) AND
          (I.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN) AND
          ((INSTR(:NRNOTAFISCALCE, I.NRNOTAFISCALCE) <> 0) OR (:NRNOTAFISCALCE = 'T')) AND
          (I.CDFILIAL    = F.CDFILIAL    ) AND
          (I.CDFILIAL    = P.CDFILIAL    ) AND
          (NOT EXISTS (SELECT 1 FROM VENDA V
                         WHERE I.CDFILIAL       = V.CDFILIAL
                          AND I.NRNOTAFISCALCE = V.NRNOTAFISCALCE
                          AND I.CDSERIENFCE    = V.CDSERIENFCE
                          AND I.DTENTRVENDA    = V.DTENTRVENDA))
        ORDER BY
          I.CDCAIXA, I.NRNOTAFISCALCE";
           
    const ADCTRANSMISSAONFCE = "
        WITH FILTRO AS (SELECT V.CDFILIAL, V. CDCAIXA, V.NRSEQVENDA,
      SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND ,0) + NVL(I.VRACRITVEND,0)) AS VRTOTALVENDA,
      V.IDSITUVENDA,V.NRACESSONFCE, V.NRINSCRCONS, V.DTENTRVENDA, V.IDSTATUSNFCE
      FROM VENDA V, ITEMVENDA I, FILIAL F, CAIXA C
      WHERE ((INSTR(:CDFILIAL, F.CDFILIAL ) <> 0) OR (:CDFILIAL='T'))
        AND ((INSTR(:CDCAIXA, C.CDFILIAL||C.CDCAIXA) <> 0) OR (:CDCAIXA='T'))
        AND F.CDFILIAL     = V.CDFILIAL
        AND F.CDFILIAL     = C.CDFILIAL
        AND C.CDCAIXA      = V.CDCAIXA
        AND V.CDFILIAL     = I.CDFILIAL
        AND V.CDCAIXA      = I.CDCAIXA
        AND V.NRSEQVENDA   = I.NRSEQVENDA
        AND V.IDTPEMISVEND = 'N'
        AND V.IDSITUVENDA <> 'D'
        AND V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
        AND V.IDTPAMBNFCE  = :IDTPAMBNFCE
        AND V.NRACESSONFCE IS NOT NULL
        GROUP BY V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.IDSITUVENDA,V.NRACESSONFCE, V.NRINSCRCONS, V.DTENTRVENDA, IDSTATUSNFCE)
        
        SELECT SUBSTR('Nota Fiscal: ' || V.NRNOTAFISCALCE || ' Caixa: '|| V.CDCAIXA, 1, 33) AS NOTA, 'Abrir .txt' AS XMLNOTA,
              V.CDFILIAL, V.CDCAIXA, V.CDLOJA, V.NRSEQVENDA, V.DTENTRVENDA, V.NRNOTAFISCALCE, V.DTEMISSAONFCE, V.NRRECIBONFCE,
              F.NMFILIAL, V.CDSERIENFCE, V.IDSITUVENDA, V.IDTPAMBNFCE, 'U' AS IDCHECK, V.IDSTATUSNFCE, V.CDEMPRESA, V.IDMODOPERACNFCE,
              C.NMCAIXA, V.DSOBSSTATUSNFCE, V.NRACESSONFCE, F.CDINSCMUNI, F.CDINSCESTA, V.NRLANCTONFCE, P.NRCERTDIGNFCE, V.DTHRPROTOCONFCE,
              V.IDXMLVALIDNFCE, V.NRINSCRCONS, V.DSARQXMLNFCE, A.VRTOTALVENDA, S.SGESTADO, S.CDESTADOIBGE , 
              V.DSQRCODENFCE, P.CDVERSAOXMLNFCE, V.NRENVIOSEFAZ
          FROM VENDA V, PARAVEND P, ENDEFILI E, ESTADO S, FILTRO A, FILIAL F, CAIXA C
         WHERE F.CDFILIAL     = A.CDFILIAL
          AND F.CDFILIAL     = C.CDFILIAL
          AND A.CDCAIXA      = C.CDCAIXA
          AND A.CDFILIAL     = V.CDFILIAL
          AND A.CDCAIXA      = V.CDCAIXA
          AND A.NRSEQVENDA   = V.NRSEQVENDA
          AND V.CDFILIAL     = P.CDFILIAL
          AND E.CDFILIAL     = A.CDFILIAL
          AND E.IDTPENDEFILI = 'P'
          AND S.CDPAIS       = E.CDPAIS
          AND S.SGESTADO     = E.SGESTADO
          AND (INSTR(:SITUACAO, A.IDSTATUSNFCE) <> 0)
          AND ((INSTR(:IDMODOPERACNFCE, SUBSTR(A.NRACESSONFCE,35,1)) <> 0) OR (:IDMODOPERACNFCE='T'))
          AND ((INSTR(:IDSITUVENDA, A.IDSITUVENDA) <> 0) OR (:IDSITUVENDA='T'))
          AND ((INSTR(:NRINSCRCONS, A.NRINSCRCONS) <> 0) OR (:NRINSCRCONS='T'))
      ORDER BY V.CDFILIAL, V.CDCAIXA, V.CDSERIENFCE, V.NRNOTAFISCALCE";
        
    const ACMUPDCHAVEACESSO = "
        UPDATE VENDA
          SET NRACESSONFCE = :NRACESSONFCE, IDXMLVALIDNFCE = 'N'
         WHERE (CDFILIAL   = :CDFILIAL  ) AND
              (CDCAIXA    = :CDCAIXA   ) AND
              (NRSEQVENDA = :NRSEQVENDA)";
               
    /* DTEMISSAONFCE = SYSDATE */
    
    const ACMUPDCHAVEACESSOSP = "
        UPDATE VENDA
          SET NRACESSONFCE = :NRACESSONFCE,
              IDXMLVALIDNFCE = 'N'
         WHERE (CDFILIAL   = :CDFILIAL  ) AND
              (CDCAIXA    = :CDCAIXA   ) AND
              (NRSEQVENDA = :NRSEQVENDA)";        
               
    const ADQTRANSMISSAONFCE = "SELECT * FROM (SELECT
  FI.CDFILIAL, FI.NMFILIAL, FI.NMRAZSOCFILI, FI.NRINSJURFILI, FI.CDINSCMUNI, FI.CDINSCESTA,
  FI.IDCUMUPISCOFIL, CX.CDCAIXA, CX.NMCAIXA, CX.CDLOJA,
  LJ.IDCOMISVENDA, LJ.VRCOMISVENDA, LJ.IDIMPCNPJCLIE,
  IE.SGESTADO, IE.DSENDEIEST, IE.NMBAIRIEST, IE.NRCEPIEST,
  EM.CDCNAE, PE.CDSITUCRT, PV.IDAMBTRABNFCE, PV.NRCERTDIGNFCE, PV.CDURLWSNFC, PV.CDIDTOKENPROD,
  PV.CDIDTOKENHOMO, PV.CDCODSCONSPROD, PV.CDCODSCONSHOMO, PV.CDVERSAOXMLNFCE, PV.CDVERSAOQRCNFCE,
  ES.CDESTADOIBGE, ES.HRDIFUTCFUSO, MN.CDMUNICIBGE, MN.NMMUNICIPIO,
  VE. NRSEQVENDA, TO_CHAR(VE.DTEMISSAONFCE, 'YYYY-MM-DD HH:MM:SS') AS DTEMISSAONFCE, VE.NRNOTAFISCALCE, VE.NRLANCTONFCE, VE.NRACESSONFCE,
  VE.IDTPAMBNFCE, VE.CDSERIENFCE, VE.IDORIGEMVENDA, VE.VRDESCVENDA, VE.IDSTATUSNFCE, VE.IDSITUVENDA,
  VE.CDCLIENTE, VE.CDCONSUMIDOR, VE.DSENDECONSVENDA,
  SUM(IT.QTPRODVEND) AS QTPRODVEND,

  (IT.VRUNITVEND+IT.VRUNITVENDCL +  SUM(NVL((case
                                                  when (LJ.CDPRODTAXASERV = PR.CDPRODUTO) or
                                 						     (LJ.CDPRODTAXAENTR = PR.CDPRODUTO) then IT.VRACRITVEND
                                                  else 0
                                                  end), 0))) AS VRUNITVEND,
  SUM(NVL((case
					   when (LJ.CDPRODTAXASERV = PR.CDPRODUTO) or
						     (LJ.CDPRODTAXAENTR = PR.CDPRODUTO) then 0
				   else
					   IT.VRACRITVEND
				   end), 0)) AS VRACRITVEND,

  SUM(NVL(IT.VRDESITVEND,0)) AS VRDESITVEND,
  PR.CDPRODUTO, PR.SGUNIDADE, PR.NMPRODUTO, PR.CDBARPRODUTO, PR.CDCLASFISC,
  (NVL(
  CASE PV.IDENVIACDBARXML
  WHEN 'S' THEN PR.CDBARPRODUTO
  WHEN 'X' THEN PR.CDPRODUTOEXT
  ELSE PR.CDARVPROD END
  ,PR.CDARVPROD)) AS CDARVPROD,
  AL.VRPEALIMPFIS, AL.IDTPIMPOSFIS, AL.VRALIQIBPT, AL.VRALIQIBPTES, AL.CDCFOPPFIS,
  AL.CDCSTICMS, AL.CDCSTPISCOF, AL.VRALIQPIS, AL.VRALIQCOFINS, AL.VRPERCREDUCAO,
  NVL(AL.CDCEST,PR.CDCEST) AS CDCEST, AL.VRPEALIMPFISEF, AL.VRPERCREDUCAOEF,
  IP.VRTOTTRIBIBPT, '' AS NRSEQUITVEND, AL.CDCBENEF, IP.VRBASECALCICMS, IP.VRBCPISCOFINS,
  IP.VRBASECALCREDUZ,
  ROUND_ABNT(SUM(TRUNC(IT.QTPRODVEND * (IT.VRUNITVEND + NVL(IT.VRUNITVENDCL,0)),2) + NVL(IT.VRACRITVEND,0) - NVL(IT.VRDESITVEND,0)) * AL.VRPEALIMPFIS / 100, 2) AS VRIMPOPRODIT,
  SUM(NVL(IP.VRIMPOPRODREDUZ,0)) AS VRIMPOPRODREDUZ,
  IP.VRIMPPIS, IP.VRIMPCOFINS, IP.VRIMPICMSFCP, AL.VRALIQFCP,
  PV.IDRESPTECNFCE, IP.VRPEALEFPRODIT, IP.VRIMPOEFPRODIT, IP.VRBCREDUZEFICMS, IP.VRPRBCREDUEFICMS,
  AL.VRALIQICMSDES, TRIM(REPLACE(REPLACE(CX.DSOBSCUPFIS,CHR(10),''),CHR(13),'')) AS DSOBSCUPFIS , FI.NRORG, PA.CDPAISBACEN, PA.NMPAIS, IE.CDPAIS,
  EM.CDEMPRESA, MN.CDMUNICIPIO, PV.NMARQCERTNFCE, PV.DSSENHACERTNFCE, IP.CDCSTPRODI, IP.VRPEALPRODIT, IP.VRPERPIS
FROM
  VENDA VE, FILIAL FI, CAIXA CX, LOJA LJ, INSCRESTAD IE, EMPRESA EM, PARAMEMPRESA PE,
  PARAVEND PV, ESTADO ES, MUNICIPIO MN, ITEMVENDA IT,
  (select DISTINCT PX.CDPRODUTOEXT, PR.* FROM PRODUTO PR LEFT JOIN PRODEXTERNO PX ON PX.CDPRODUTO=PR.CDPRODUTO AND  PX.CDFILIAL = :CDFILIAL) PR,
    ALIQIMPFIS AL, ITVENDAIMPOS IP, PAIS PA
WHERE
  (FI.CDFILIAL      = :CDFILIAL    ) AND
  (CX.CDCAIXA       = :CDCAIXA     ) AND
  (LJ.CDLOJA        = :CDLOJA      ) AND
  (FI.CDFILIAL      = CX.CDFILIAL    ) AND
  (FI.CDFILIAL      = LJ.CDFILIAL    ) AND
  (FI.CDEMPRESA     = EM.CDEMPRESA   ) AND
  (FI.CDEMPRESA     = PE.CDEMPRESA   ) AND
  (FI.CDFILIAL      = PV.CDFILIAL    ) AND
  (IE.CDINSCESTA    = FI.CDINSCESTA  ) AND
  (IE.CDEMPRESA     = FI.CDEMPRESA   ) AND
  (IE.CDPAIS        = ES.CDPAIS      ) AND
  (IE.SGESTADO      = ES.SGESTADO    ) AND
  (IE.CDPAIS        = PA.CDPAIS      ) AND
  (IE.CDPAIS        = MN.CDPAIS      ) AND
  (IE.SGESTADO      = MN.SGESTADO    ) AND
  (IE.CDMUNICIPIO   = MN.CDMUNICIPIO ) AND
  (FI.CDFILIAL      = VE.CDFILIAL    ) AND
  (CX.CDCAIXA       = VE.CDCAIXA     ) AND
  (VE.NRSEQVENDA    = :NRSEQVENDA  ) AND
  (VE.NRSEQVENDA    = IT.NRSEQVENDA  ) AND
  (VE.CDFILIAL      = IT.CDFILIAL    ) AND
  (VE.CDCAIXA       = IT.CDCAIXA     ) AND
  (IT.CDPRODUTO     = PR.CDPRODUTO   ) AND
  (AL.CDPRODUTO     = PR.CDPRODUTO   ) AND
  (FI.CDFILIAL      = AL.CDFILIAL    ) AND
  (IP.CDFILIAL      = VE.CDFILIAL    ) AND
  (IP.CDCAIXA       = VE.CDCAIXA     ) AND
  (IP.NRSEQUITVEND  = IT.NRSEQUITVEND) AND
  (IP.NRSEQVENDA    = VE.NRSEQVENDA  ) AND
  (VE.IDSTATUSNFCE  <> 'T'           ) AND
  (VE.NRACESSONFCE IS NOT NULL       ) AND
  (PV.IDAGRUPITEMXML = 'S'           ) AND
  ('I' = (CASE IT.QTPRODVEND-TRUNC(IT.QTPRODVEND) WHEN 0 THEN 'I' ELSE 'D' END))
GROUP BY
  FI.CDFILIAL, FI.NMFILIAL, FI.NMRAZSOCFILI, FI.NRINSJURFILI, FI.CDINSCMUNI, FI.CDINSCESTA, FI.IDCUMUPISCOFIL,
  CX.CDCAIXA, CX.NMCAIXA, CX.CDLOJA,
  LJ.IDCOMISVENDA, LJ.VRCOMISVENDA, LJ.IDIMPCNPJCLIE,
  IE.SGESTADO, IE.DSENDEIEST, IE.NMBAIRIEST, IE.NRCEPIEST,
  EM.CDCNAE,
  PE.CDSITUCRT,
  PV.IDAMBTRABNFCE, PV.NRCERTDIGNFCE, PV.CDURLWSNFC, PV.CDIDTOKENPROD, PV.CDIDTOKENHOMO, PV.CDCODSCONSPROD, PV.CDCODSCONSHOMO,  PV.CDVERSAOXMLNFCE, PV.CDVERSAOQRCNFCE,
  ES.CDESTADOIBGE, ES.HRDIFUTCFUSO,
  MN.CDMUNICIBGE, MN.NMMUNICIPIO,
  VE. NRSEQVENDA, VE.DTEMISSAONFCE, VE.NRNOTAFISCALCE, VE.NRLANCTONFCE, VE.NRACESSONFCE, VE.IDTPAMBNFCE,
  VE.CDSERIENFCE, VE.IDORIGEMVENDA, VE.VRDESCVENDA, VE.IDSTATUSNFCE, VE.IDSITUVENDA,
  VE.CDCLIENTE, VE.CDCONSUMIDOR, VE.DSENDECONSVENDA,
  (IT.VRUNITVEND+IT.VRUNITVENDCL), PR.CDPRODUTO, PR.SGUNIDADE, PR.NMPRODUTO, PR.CDBARPRODUTO, PR.CDCLASFISC,
  (NVL(
  CASE PV.IDENVIACDBARXML
  WHEN 'S' THEN PR.CDBARPRODUTO
  WHEN 'X' THEN PR.CDPRODUTOEXT
  ELSE PR.CDARVPROD END
  ,PR.CDARVPROD)),
  AL.VRPEALIMPFIS, AL.IDTPIMPOSFIS, AL.VRALIQIBPT, AL.VRALIQIBPTES, AL.CDCFOPPFIS, AL.CDCSTICMS, AL.CDCSTPISCOF,
  AL.VRALIQPIS, AL.VRALIQCOFINS, AL.VRPERCREDUCAO, NVL(AL.CDCEST,PR.CDCEST), AL.VRPEALIMPFISEF, AL.VRPERCREDUCAOEF,
  IP.VRTOTTRIBIBPT, AL.CDCBENEF, IP.VRBASECALCICMS, IP.VRBCPISCOFINS, IP.VRBASECALCREDUZ,
  IP.VRIMPPIS, IP.VRIMPCOFINS, IP.VRIMPICMSFCP, AL.VRALIQFCP, PV.IDRESPTECNFCE,
  IP.VRPEALEFPRODIT, IP.VRIMPOEFPRODIT, IP.VRBCREDUZEFICMS, IP.VRPRBCREDUEFICMS,
  AL.VRALIQICMSDES, CX.DSOBSCUPFIS, FI.NRORG, PA.CDPAISBACEN, PA.NMPAIS, IE.CDPAIS,EM.CDEMPRESA, MN.CDMUNICIPIO,
  PV.NMARQCERTNFCE, PV.DSSENHACERTNFCE, IP.CDCSTPRODI, IP.VRPEALPRODIT, IP.VRPERPIS

UNION

SELECT
  FI.CDFILIAL, FI.NMFILIAL, FI.NMRAZSOCFILI, FI.NRINSJURFILI, FI.CDINSCMUNI, FI.CDINSCESTA,
  FI.IDCUMUPISCOFIL, CX.CDCAIXA, CX.NMCAIXA, CX.CDLOJA,
  LJ.IDCOMISVENDA, LJ.VRCOMISVENDA, LJ.IDIMPCNPJCLIE,
  IE.SGESTADO, IE.DSENDEIEST, IE.NMBAIRIEST, IE.NRCEPIEST, EM.CDCNAE, PE.CDSITUCRT,
  PV.IDAMBTRABNFCE, PV.NRCERTDIGNFCE, PV.CDURLWSNFC, PV.CDIDTOKENPROD, PV.CDIDTOKENHOMO,
  PV.CDCODSCONSPROD, PV.CDCODSCONSHOMO, PV.CDVERSAOXMLNFCE, PV.CDVERSAOQRCNFCE,
  ES.CDESTADOIBGE, ES.HRDIFUTCFUSO, MN.CDMUNICIBGE, MN.NMMUNICIPIO,
  VE. NRSEQVENDA, TO_CHAR(VE.DTEMISSAONFCE, 'YYYY-MM-DD HH:MM:SS') AS DTEMISSAONFCE, VE.NRNOTAFISCALCE, VE.NRLANCTONFCE,
  VE.NRACESSONFCE, VE.IDTPAMBNFCE,
  VE.CDSERIENFCE, VE.IDORIGEMVENDA, VE.VRDESCVENDA, VE.IDSTATUSNFCE, VE.IDSITUVENDA,
  VE.CDCLIENTE, VE.CDCONSUMIDOR, VE.DSENDECONSVENDA,
  IT.QTPRODVEND,

  (IT.VRUNITVEND+IT.VRUNITVENDCL + NVL((case
                                             when (LJ.CDPRODTAXASERV = PR.CDPRODUTO) or
                             						     (LJ.CDPRODTAXAENTR = PR.CDPRODUTO) then IT.VRACRITVEND
                                             else 0
                                              end), 0)) AS VRUNITVEND,
  NVL((case
					   when (LJ.CDPRODTAXASERV = PR.CDPRODUTO) or
						     (LJ.CDPRODTAXAENTR = PR.CDPRODUTO) then 0
				   else
					   IT.VRACRITVEND
				   end), 0) AS VRACRITVEND,

  NVL(IT.VRDESITVEND,0) AS VRDESITVEND,
  PR.CDPRODUTO, PR.SGUNIDADE, PR.NMPRODUTO, PR.CDBARPRODUTO, PR.CDCLASFISC,
  (NVL(
  CASE PV.IDENVIACDBARXML
  WHEN 'S' THEN PR.CDBARPRODUTO
  WHEN 'X' THEN PR.CDPRODUTOEXT
  ELSE PR.CDARVPROD END
  ,PR.CDARVPROD)) AS CDARVPROD,
  AL.VRPEALIMPFIS, AL.IDTPIMPOSFIS, AL.VRALIQIBPT, AL.VRALIQIBPTES, AL.CDCFOPPFIS,
  AL.CDCSTICMS, AL.CDCSTPISCOF, AL.VRALIQPIS, AL.VRALIQCOFINS, AL.VRPERCREDUCAO,
  NVL(AL.CDCEST,PR.CDCEST) AS CDCEST, AL.VRPEALIMPFISEF, AL.VRPERCREDUCAOEF,
  IP.VRTOTTRIBIBPT, IT.NRSEQUITVEND, AL.CDCBENEF, IP.VRBASECALCICMS, IP.VRBCPISCOFINS,
  IP.VRBASECALCREDUZ, IP.VRIMPOPRODIT, IP.VRIMPOPRODREDUZ,
  IP.VRIMPPIS,IP.VRIMPCOFINS, IP.VRIMPICMSFCP, AL.VRALIQFCP,
  PV.IDRESPTECNFCE, IP.VRPEALEFPRODIT, IP.VRIMPOEFPRODIT, IP.VRBCREDUZEFICMS, IP.VRPRBCREDUEFICMS,
  AL.VRALIQICMSDES, TRIM(REPLACE(REPLACE(CX.DSOBSCUPFIS,CHR(10),''),CHR(13),'')) AS DSOBSCUPFIS, FI.NRORG, PA.CDPAISBACEN, PA.NMPAIS, IE.CDPAIS, EM.CDEMPRESA, MN.CDMUNICIPIO,
  PV.NMARQCERTNFCE, PV.DSSENHACERTNFCE, IP.CDCSTPRODI, IP.VRPEALPRODIT, IP.VRPERPIS
FROM
  VENDA VE, FILIAL FI, CAIXA CX, LOJA LJ, INSCRESTAD IE, EMPRESA EM, PARAMEMPRESA PE, PARAVEND PV, ESTADO ES,
  MUNICIPIO MN, ITEMVENDA IT,
  (select DISTINCT PX.CDPRODUTOEXT, PR.* FROM PRODUTO PR LEFT JOIN PRODEXTERNO PX ON PX.CDPRODUTO=PR.CDPRODUTO AND  PX.CDFILIAL = :CDFILIAL) PR,
  ALIQIMPFIS AL, ITVENDAIMPOS IP, PAIS PA
WHERE
  (FI.CDFILIAL      = :CDFILIAL    ) AND
  (CX.CDCAIXA       = :CDCAIXA     ) AND
  (LJ.CDLOJA        = :CDLOJA      ) AND
  (FI.CDFILIAL      = CX.CDFILIAL    ) AND
  (FI.CDFILIAL      = LJ.CDFILIAL    ) AND
  (FI.CDEMPRESA     = EM.CDEMPRESA   ) AND
  (FI.CDEMPRESA     = PE.CDEMPRESA   ) AND
  (FI.CDFILIAL      = PV.CDFILIAL    ) AND
  (IE.CDINSCESTA    = FI.CDINSCESTA  ) AND
  (IE.CDEMPRESA     = FI.CDEMPRESA   ) AND
  (IE.CDPAIS        = ES.CDPAIS      ) AND
  (IE.SGESTADO      = ES.SGESTADO    ) AND
  (IE.CDPAIS        = PA.CDPAIS      ) AND
  (IE.CDPAIS        = MN.CDPAIS      ) AND
  (IE.SGESTADO      = MN.SGESTADO    ) AND
  (IE.CDMUNICIPIO   = MN.CDMUNICIPIO ) AND
  (FI.CDFILIAL      = VE.CDFILIAL    ) AND
  (CX.CDCAIXA       = VE.CDCAIXA     ) AND
  (VE.NRSEQVENDA    = :NRSEQVENDA  ) AND
  (VE.NRSEQVENDA    = IT.NRSEQVENDA  ) AND
  (VE.CDFILIAL      = IT.CDFILIAL    ) AND
  (VE.CDCAIXA       = IT.CDCAIXA     ) AND
  (IT.CDPRODUTO     = PR.CDPRODUTO   ) AND
  (AL.CDPRODUTO     = PR.CDPRODUTO   ) AND
  (FI.CDFILIAL      = AL.CDFILIAL    ) AND
  (IP.CDFILIAL      = VE.CDFILIAL    ) AND
  (IP.CDCAIXA       = VE.CDCAIXA     ) AND
  (IP.NRSEQUITVEND  = IT.NRSEQUITVEND) AND
  (IP.NRSEQVENDA    = VE.NRSEQVENDA  ) AND
  (VE.IDSTATUSNFCE  <> 'T'           ) AND
  (VE.NRACESSONFCE IS NOT NULL       ) AND
  (('D' = (CASE IT.QTPRODVEND-TRUNC(IT.QTPRODVEND) WHEN 0 THEN 'I' ELSE 'D' END)) OR (PV.IDAGRUPITEMXML = 'N')))
  ORDER BY CDPRODUTO DESC";
           
    const ADQCONSUMIDORNFCE = "
    SELECT
      VE.CDFILIAL, VE.CDCAIXA, VE.NRSEQVENDA, VE.DTENTRVENDA, VE.NRNOTAFISCALCE, VE.NRACESSONFCE,
      VE.NRINSCRCONS,
      REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(VE.NMCONSVEND,'.',''),',',''),'(',''),')',''),'-',''),'/',''),'*',''),'\"',''),'\','') AS NMCONSVEND,
      VE.CDCLIENTE, VE.CDCONSUMIDOR,
      REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(CO.NMCONSUMIDOR,'.',''),',',''),'(',''),')',''),'-',''),'/',''),'*',''),'\"',''),'\','') AS NMCONSUMIDOR,
      CO.NRCPFRESPCON, CL.IDTPIJURCLIE, CL.NRINSJURCLIE
    FROM
      CLIENTE CL, VENDA VE LEFT JOIN CONSUMIDOR CO
    	ON VE.CDCLIENTE = CO.CDCLIENTE AND VE.CDCONSUMIDOR = CO.CDCONSUMIDOR
    WHERE
      (VE.CDFILIAL   = :CDFILIAL  ) AND
      (VE.CDCAIXA    = :CDCAIXA   ) AND
      (VE.NRSEQVENDA = :NRSEQVENDA) AND
      (VE.CDCLIENTE  = CL.CDCLIENTE )";
       
    const ADQRECCANCNFCE = "
    SELECT I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA, ' ' AS IDTIPORECE, ' ' AS IDTIPOMOVIVE,
          SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)),2) - NVL(I.VRDESITVEND ,0) + NVL(I.VRACRITVEND,0)) AS VRMOVIVEND
      FROM ITEMVENDA I
     WHERE I.CDFILIAL   = :CDFILIAL
      AND I.CDCAIXA    = :CDCAIXA
      AND I.NRSEQVENDA = :NRSEQVENDA
    GROUP BY I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA";
    
    const ADQRECEBNFCE310 = "
    SELECT TR.NMTIPORECE, (CASE MC.NRSEQUMOVI WHEN SA.NRSEQUMOVI THEN MC.VRMOVIVEND-NVL(S.VRMOVIVEND,0) ELSE MC.VRMOVIVEND END) AS VRMOVIVEND,
          TR.IDTIPORECE, TR.CDBANCARTCR, TR.CDADMINCART, MC.CDNSUHOSTTEF, MC.QTPARCRECEB, TR.CDBANCARTSEFAZ, TR.IDDESABTEF
      FROM MOVCAIXA MC, TIPORECE TR, (SELECT M.VRMOVIVEND, MAX(M.NRSEQUMOVI) AS NRSEQUMOVI, M.NRSEQVENDA
                                        FROM MOVCAIXA M
                                      WHERE M.CDFILIAL  =:CDFILIAL   
                                         AND M.CDCAIXA   =:CDCAIXA
                                         AND M.NRSEQVENDA=:NRSEQVENDA
                                         AND M.IDTIPOMOVIVE IN ('E','T')                                     
                                         AND M.VRMOVIVEND = (SELECT MAX(VRMOVIVEND)
                                                              FROM MOVCAIXA
                                                              WHERE CDFILIAL  =:CDFILIAL   
                                                                AND CDCAIXA   =:CDCAIXA
                                                                AND NRSEQVENDA=:NRSEQVENDA
                                                                AND IDTIPOMOVIVE IN ('E','T'))
                                      GROUP BY M.VRMOVIVEND, M.NRSEQVENDA)SA,
                                     (SELECT CDFILIAL, CDCAIXA, NRSEQVENDA, VRMOVIVEND
                                        FROM MOVCAIXA
                                      WHERE CDFILIAL    =:CDFILIAL
                                         AND CDCAIXA     =:CDCAIXA
                                         AND NRSEQVENDA  =:NRSEQVENDA
                                         AND IDTIPOMOVIVE='S' )S
     WHERE MC.CDFILIAL  =:CDFILIAL
      AND MC.CDCAIXA   =:CDCAIXA
      AND MC.NRSEQVENDA=:NRSEQVENDA
      AND TR.CDTIPORECE=MC.CDTIPORECE
      AND SA.NRSEQVENDA=S.NRSEQVENDA(+)
      AND MC.IDTIPOMOVIVE IN ('E','T')";
       
    const ADQRECEBIMENTONFCE = "
    SELECT TR.NMTIPORECE, MC.VRMOVIVEND, TR.IDTIPORECE, MC.IDTIPOMOVIVE, TR.CDBANCARTCR, TR.CDADMINCART, MC.CDNSUHOSTTEF, MC.QTPARCRECEB, TR.CDBANCARTSEFAZ, TR.IDDESABTEF
      FROM TIPORECE TR, MOVCAIXA MC
     WHERE MC.CDFILIAL  =:CDFILIAL
      AND MC.CDCAIXA   =:CDCAIXA
      AND MC.NRSEQVENDA=:NRSEQVENDA
      AND TR.CDTIPORECE=MC.CDTIPORECE";
       
    const ADQGETDUPLICIDADES = "
        SELECT V.NRACESSONFCE NRCHAVENOTA, V.NRRECIBONFCE NRRECEBIDO, V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.CDLOJA, V.DTVENDA, V.CDEMPRESA, V.IDSITUVENDA, V.IDSTATUSNFCE,
              V.IDTPAMBNFCE, V.IDMODOPERACNFCE, V.NRNOTAFISCALCE, V.CDSERIENFCE, V.NRPROTOCOLONFCE, SUBSTR(NRLANCTONFCE,1,2)||SUBSTR(NRACESSONFCE,36,8) AS NRLANCTONFCE,
              SUBSTR(V.DSOBSSTATUSNFCE,1,3) AS CSTAT, V.DSOBSSTATUSNFCE,
              CASE WHEN SUBSTR(V.DSOBSSTATUSNFCE,1,3)='204' THEN V.NRACESSONFCE
                    WHEN SUBSTR(V.NRACESSONFCE,1,2   )='52'  THEN V.NRACESSONFCE
                    WHEN SUBSTR(V.NRACESSONFCE,1,2   )='51' AND SUBSTR(V.DSOBSSTATUSNFCE,1,3)='539' THEN V.NRACESSONFCE
                    ELSE SUBSTR(V.DSOBSSTATUSNFCE, CASE WHEN INSTR(V.DSOBSSTATUSNFCE,'chNFe: ') > 0 THEN INSTR(V.DSOBSSTATUSNFCE,'chNFe: ')+7
                                                        WHEN INSTR(V.DSOBSSTATUSNFCE,'chNFe:' ) > 0 THEN INSTR(V.DSOBSSTATUSNFCE,'chNFe:' )+6
                                                        WHEN INSTR(V.DSOBSSTATUSNFCE,'[ '     ) > 0 THEN INSTR(V.DSOBSSTATUSNFCE,'[ '     )+2
                                                        WHEN INSTR(V.DSOBSSTATUSNFCE,'['      ) > 0 THEN INSTR(V.DSOBSSTATUSNFCE,'['      )+1
                                                  ELSE 0 END, 44) END AS NRACESSONFCE,
              CASE WHEN SUBSTR(V.DSOBSSTATUSNFCE,1,3)='613' THEN V.NRRECIBONFCE
                    ELSE SUBSTR(V.DSOBSSTATUSNFCE, CASE WHEN INSTR(V.DSOBSSTATUSNFCE,'nRec: ' ) > 0 THEN INSTR(V.DSOBSSTATUSNFCE,'nRec: ' )+6
                                                        WHEN INSTR(V.DSOBSSTATUSNFCE,'nRec:'  ) > 0 THEN INSTR(V.DSOBSSTATUSNFCE,'nRec:'  )+5
                                                        WHEN INSTR(V.DSOBSSTATUSNFCE,'[ ',INSTR(V.DSOBSSTATUSNFCE,'[ ')+2) > 0 THEN INSTR(V.DSOBSSTATUSNFCE,'[ ',INSTR(V.DSOBSSTATUSNFCE,'[ ')+2)+2
                                                        ELSE 0 END, 15)
                    END AS NRRECIBONFCE,
              V.DTENVIONFCE, P.NRCERTDIGNFCE, V.DSARQXMLNFCE, P.CDVERSAOXMLNFCE
          FROM VENDA V, PARAVEND P
         WHERE V.CDFILIAL=P.CDFILIAL
          AND V.IDTPEMISVEND='N'
          AND V.IDSTATUSNFCE='R'
          AND (V.DTENTRVENDA BETWEEN TO_DATE(:DATAINI, 'DD/MM/YYYY') AND TO_DATE(:DATAFIN, 'DD/MM/YYYY'))
          AND ((INSTR(:CDFILIAL,V.CDFILIAL)<>0) OR (:CDFILIAL='T'))
          AND (SUBSTR(V.DSOBSSTATUSNFCE,1,3)='204' OR SUBSTR(V.DSOBSSTATUSNFCE,1,3)='539' OR SUBSTR(V.DSOBSSTATUSNFCE,1,3)='613' OR SUBSTR(V.DSOBSSTATUSNFCE, 1, 15) = 'R - Duplicidade' OR V.DSOBSSTATUSNFCE LIKE '%Duplicidade%')
        ORDER BY V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA";
        
        
    const ACMDUPLICIDADES = "
        UPDATE VENDA
          SET IDSTATUSNFCE = 'A',
              NRACESSONFCE = :NRACESSONFCE,
              NRRECIBONFCE = :NRRECIBONFCE
         WHERE CDFILIAL     = :CDFILIAL
          AND CDCAIXA      = :CDCAIXA
          AND NRSEQVENDA   = :NRSEQVENDA";    
           
    const UPDATE_VENDA_NFCE = "
        UPDATE VENDA
          SET DSARQXMLNFCE = :DSARQXMLNFCE, DTULTATU = SYSDATE, DSQRCODENFCE = :DSQRCODENFCE
        WHERE NRORG = :NRORG
            AND NRSEQVENDA = :NRSEQVENDA
            AND CDFILIAL = :CDFILIAL
            AND CDCAIXA = :CDCAIXA";  
            
    const ACMDUPLICIDADESGOIAS = "
        UPDATE VENDA
         SET IDSTATUSNFCE    = 'A',
             DSOBSSTATUSNFCE = '100 - Autorizado o uso da NFCe',
             NRRECIBONFCE    = NULL,
             NRACESSONFCE    = :NRACESSONFCE
      WHERE CDFILIAL        = :CDFILIAL
         AND CDCAIXA         = :CDCAIXA
         AND NRSEQVENDA      = :NRSEQVENDA";
         
    const ADQPARAVEND = "
        SELECT IDCANCEXTNFCE, IDGERALOGNFCE, DSDIRARQLOGNFCE, SYSDATE AS DTATUAL,
              IDCANCAUTNFCE, DSJUSTCANCNFCE, CDVERSAOXMLNFCE
          FROM PARAVEND
         WHERE CDFILIAL = :CDFILIAL";    
         
    const ACMUPDPROTOCNFCE = " 
        UPDATE VENDA
          SET IDSTATUSNFCE    = :IDSTATUSNFCE,
              DSOBSSTATUSNFCE = :DSOBSSTATUSNFCE,
              NRPROTOCOLONFCE = :NRPROTOCOLONFCE,
              DTHRPROTOCONFCE = :DTHRPROTOCONFCE,
              NRACESSONFCE    = :NRACESSONFCE,
              NRENVIOSEFAZ    = NVL(NRENVIOSEFAZ,1)+1
         WHERE (CDFILIAL   = :CDFILIAL  ) AND
              (CDCAIXA    = :CDCAIXA   ) AND
              (NRSEQVENDA = :NRSEQVENDA)";     
               
    const ACMUPDNFCE = "
        UPDATE VENDA
          SET IDSTATUSNFCE    = :IDSTATUSNFCE,
              DSOBSSTATUSNFCE = :DSOBSSTATUSNFCE,
              NRENVIOSEFAZ    = NVL(NRENVIOSEFAZ,1)+1
         WHERE CDFILIAL   = :CDFILIAL
          AND CDCAIXA    = :CDCAIXA
          AND NRSEQVENDA = :NRSEQVENDA";           

    const ADQNUMERACAO_FULL = "
        SELECT I.CDFILIAL, I.CDCAIXA, I.NRNOTAFISCALCE, I.CDSERIENFCE, I.IDTPAMBNFCE, '0' NRENVIOSEFAZ,
          F.CDEMPRESA, F.CDINSCESTA, F.CDINSCMUNI, P.NRCERTDIGNFCE, I.IDSTATUSNFCE,
          I.DSOBSINUTNFCE, P.CDVERSAOXMLNFCE, P.IDAMBTRABNFCE, P.CDCODSCONSPROD, P.CDURLWSNFC,
          P.CDIDTOKENPROD, P.CDCODSCONSHOMO, P.CDIDTOKENHOMO, F.NMRAZSOCFILI, F.NRINSJURFILI, IE.SGESTADO
        FROM
          INUTILIZANFCE I, FILIAL F, PARAVEND P, INSCRESTAD IE
        WHERE
          (I.CDFILIAL    = :CDFILIAL   ) AND
          ((INSTR(:CDCAIXA, I.CDFILIAL||I.CDCAIXA) <> 0) OR (:CDCAIXA = 'T')) AND
          (I.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN) AND
          (I.CDFILIAL    = F.CDFILIAL     ) AND
          (I.CDFILIAL    = P.CDFILIAL     ) AND
          (IE.CDINSCESTA = F.CDINSCESTA   ) AND
          (IE.CDEMPRESA  = F.CDEMPRESA    ) AND
          (NOT EXISTS (SELECT 1 FROM VENDA V
                         WHERE I.CDFILIAL       = V.CDFILIAL
                          AND I.NRNOTAFISCALCE = V.NRNOTAFISCALCE
                          AND I.CDSERIENFCE    = V.CDSERIENFCE
                          AND I.DTENTRVENDA    = V.DTENTRVENDA))
        ORDER BY
          I.CDCAIXA, I.NRNOTAFISCALCE";
           
    const ACMUPDINUTILIZANFCE = "
        UPDATE INUTILIZANFCE
          SET IDSTATUSNFCE    = :IDSTATUSNFCE,
              DTHRINUTNFCE    = :DTHRINUTNFCE,
              NRPROTOINUTNFCE = :NRPROTOINUTNFCE,
              IDTPEMISINUNFCE = :IDTPEMISINUNFCE,
              DSRAZAOINUTNFCE = :DSRAZAOINUTNFCE,
              DSOBSINUTNFCE   = :DSOBSINUTNFCE
         WHERE (CDFILIAL       = :CDFILIAL      ) AND
              (CDCAIXA        = :CDCAIXA       ) AND
              (NRNOTAFISCALCE = :NRNOTAFISCALCE)";   
               
    const ACMUPDINUTILIZA = "
        UPDATE INUTILIZANFCE
          SET IDSTATUSNFCE    = :IDSTATUSNFCE,
              IDTPEMISINUNFCE = :IDTPEMISINUNFCE,
              DSRAZAOINUTNFCE = :DSRAZAOINUTNFCE,
              DSOBSINUTNFCE   = :DSOBSINUTNFCE
         WHERE (CDFILIAL       = :CDFILIAL      ) AND
              (CDCAIXA        = :CDCAIXA       ) AND
              (NRNOTAFISCALCE = :NRNOTAFISCALCE)";
               
    const UPDATENRINSCRCONS  = "
        UPDATE VENDA
          SET NRINSCRCONS     = :NRINSCRCONS
         WHERE (CDFILIAL       = :CDFILIAL      ) AND
              (CDCAIXA        = :CDCAIXA       ) AND
              (NRSEQVENDA     = :NRSEQVENDA    ) AND
              (NRNOTAFISCALCE = :NRNOTAFISCALCE)";
               
    const UPDATENRENVIOSEFAZ  = "
        UPDATE VENDA 
            SET
            NRENVIOSEFAZ = 1,
            IDCONSUINDEVIDO = 'N'
        WHERE (CDFILIAL = :CDFILIAL)
            AND (CDCAIXA = :CDCAIXA)
            AND (NRSEQVENDA = :NRSEQVENDA)
            AND (NRORG = :NRORG)";
        
}